<?php

namespace BlueSpice\Bookshelf;

use Title;
use TitleFactory;

class TreeDataProvider {

	/** @var TitleFactory */
	private $titleFactory;

	/** @var Title */
	private $title;

	/** @var BookContextProviderFactory */
	private $bookContextProviderFactory = null;

	/** @var BookLookup */
	private $bookLookup = null;

	/**
	 * @param BookLookup $bookLookup
	 * @param BookContextProviderFactory $bookContextProviderFactory
	 * @param TitleFactory $titleFactory
	 */
	public function __construct(
		BookLookup $bookLookup,
		BookContextProviderFactory $bookContextProviderFactory, TitleFactory $titleFactory
	) {
		$this->bookLookup = $bookLookup;
		$this->bookContextProviderFactory = $bookContextProviderFactory;
		$this->titleFactory = $titleFactory;
	}

	/**
	 * @param Title $title
	 * @return array
	 */
	public function get( Title $title ): array {
		$this->title = $title;

		$books = $this->bookLookup->getBooksForPage( $title );
		if ( empty( $books ) ) {
			return [];
		}

		$bookContextProvider = $this->bookContextProviderFactory->getProvider( $title );
		$activeBook = $bookContextProvider->getActiveBook();
		if ( !$activeBook ) {
			return [];
		}
		$items = $this->bookLookup->getBookHierarchy( $activeBook );

		$data = [];
		foreach ( $items as $item ) {
			$this->processItem( $item, $data );
		}

		return $data;
	}

	/**
	 * @param array $item
	 * @param array &$data
	 * @param string $path
	 * @return void
	 */
	private function processItem( array $item, array &$data, $path = '' ): void {
		$itemData = [];

		$fullId = md5( $item['chapter_name'] );
		$id = substr( $fullId, 0, 6 );
		$path .= "/$id";

		$title = $this->titleFactory->makeTitle(
			$item['chapter_namespace'],
			$item['chapter_title']
		);

		if ( $this->getNodeType( $item ) === 'plain-text' ) {
			$itemData = $this->makeTextNode( $item, $title, $id, $path );
		}
		if ( $this->getNodeType( $item ) === 'wikilink-with-alias' ) {
			$itemData = $this->makeLinkNode( $item, $title, $id, $path );
		}
		if ( $this->getNodeType( $item ) === 'wikilink' ) {
			$itemData = $this->makeLinkNode( $item, $title, $id, $path );
		}

		if ( isset( $item['chapter_children'] ) && !empty( $item['chapter_children'] ) ) {
			foreach ( $item['chapter_children'] as $children ) {
				$this->processItem( $children, $itemData['items'], $path );
			}
		}

		if ( !empty( $itemData ) ) {
			$data[] = $itemData;
		}
	}

	/**
	 * @param array $item
	 * @return string
	 */
	private function getNodeType( array $item ): string {
		return $item['chapter_type'];
	}

	/**
	 * @param array $item
	 * @param Title $title
	 * @param string $id
	 * @param string $path
	 * @return array
	 */
	private function makeTextNode( array $item, Title $title, string $id, string $path ): array {
		$data = [
			'id' => $id,
			'name' => $item['chapter_name'],
			'text' => $item['chapter_number'] . ' ' . $item['chapter_name'],
			'path' => trim( $path, '/' ),
			'items' => []
		];

		$classes = $this->getClasses( $title, [ $item['chapter_type'] ] );

		if ( !empty( $classes ) ) {
			$data['classes'] = $classes;
		}

		return $data;
	}

	/**
	 * @param array $item
	 * @param Title $title
	 * @param string $id
	 * @param string $path
	 * @return array
	 */
	private function makeLinkNode( array $item, Title $title, $id, $path ): array {
		$data = $this->makeTextNode( $item, $title, $id, $path );
		$data['href'] = $title->getLocalURL();

		return $data;
	}

	/**
	 * @param Title $title
	 * @return bool
	 */
	private function isActiveTitle( Title $title ): bool {
		if ( $title->equals( $this->title ) ) {
			return true;
		}

		return false;
	}

	/**
	 * @param Title $title
	 * @param array $classes
	 * @return array
	 */
	private function getClasses( Title $title, array $classes = [] ): array {
		if ( $this->isActiveTitle( $title ) ) {
			$classes[] = 'acitve';
		}

		return $classes;
	}

}
